package gpl.pierrick.brihaye.aramorph;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class contains a main method that transforms dictionaries and
 * compatability tables from text format to XML format.
 * @author Ahmed Saad
 */
public class DictTables2XML {
	/** A pattern which is used in parsing the dictionary file*/
	private static Pattern p = Pattern.compile(".*" + "<pos>(.+?)</pos>" + ".*");
	/**This map is used for translation of dictionaries*/
	private static Map translationMap = getTranslationSet();
	//private static final String XML_VERSION = "1.0";
	private static final String XML_ENCODING = "UTF-8";

	private static final String ROOT = "root";
	private static final String VALUE = "value";
	private static final String ENTRY = "entry";
	private static final String LEMMA = "lemma";
	private static final String ID = "id";
	private static final String UNVOCALIZED = "unvocalized";
	private static final String VOCALIZED = "vocalized";
	private static final String MORPHOLOGICAL_CATEGORY = "morphological-category";
	private static final String GLOSSES = "glosses";
	private static final String GLOSS = "gloss";
	private static final String GRAMMATICAL_CATEGORIES = "grammatical-categories";
	private static final String GRAMMATICAL_CATEGORY = "grammatical-category";
	//private static final String LEMMA_ID = "lemmaID";
	

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new DictTables2XML();
	}
	
	public DictTables2XML() {
		convertAll();
	}
	
	public void convertAll() {
		System.out.println("Transforming dictionaries and compatability tables to XML...");
		// transform 3 lexicons
		
		transformDictionary2("dictPrefixes", this.getClass().getResourceAsStream("dictionaries/dictPrefixes"));
		transformDictionary2("dictStems", this.getClass().getResourceAsStream("dictionaries/dictStems"));
		transformDictionary2("dictSuffixes", this.getClass().getResourceAsStream("dictionaries/dictSuffixes"));
		//transform 3 compatibility tables
		transformCompatibilityTable("prefix","stem","tableAB", this.getClass().getResourceAsStream("dictionaries/tableAB"));
		transformCompatibilityTable("prefix","suffix","tableAC", this.getClass().getResourceAsStream("dictionaries/tableAC"));
		transformCompatibilityTable("stem","suffix","tableBC", this.getClass().getResourceAsStream("dictionaries/tableBC"));
		
		//write status message
		System.out.println("... done.");
	}
	
	private static Map getTranslationSet() {
		Map s = new HashMap(61);
		
		s.put(new Character(';'),new Character('/')); //TODO : is it necessary ?
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('A'));
		s.put(new Character(''),new Character('C'));
		s.put(new Character(''),new Character('E'));
		s.put(new Character(''),new Character('E'));
		s.put(new Character(''),new Character('E'));
		s.put(new Character(''),new Character('E'));
		s.put(new Character(''),new Character('I'));
		s.put(new Character(''),new Character('I'));
		s.put(new Character(''),new Character('I'));
		s.put(new Character(''),new Character('I'));
		s.put(new Character(''),new Character('N'));
		s.put(new Character(''),new Character('O'));
		s.put(new Character(''),new Character('O'));
		s.put(new Character(''),new Character('O'));
		s.put(new Character(''),new Character('O'));
		s.put(new Character(''),new Character('O'));
		s.put(new Character(''),new Character('U'));
		s.put(new Character(''),new Character('U'));
		s.put(new Character(''),new Character('U'));
		s.put(new Character(''),new Character('U'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('a'));
		s.put(new Character(''),new Character('c'));
		s.put(new Character(''),new Character('e'));
		s.put(new Character(''),new Character('e'));
		s.put(new Character(''),new Character('e'));
		s.put(new Character(''),new Character('e'));
		s.put(new Character(''),new Character('i'));
		s.put(new Character(''),new Character('i'));
		s.put(new Character(''),new Character('i'));
		s.put(new Character(''),new Character('i'));
		s.put(new Character(''),new Character('n'));
		s.put(new Character(''),new Character('o'));
		s.put(new Character(''),new Character('o'));
		s.put(new Character(''),new Character('o'));
		s.put(new Character(''),new Character('o'));
		s.put(new Character(''),new Character('o'));
		s.put(new Character(''),new Character('u'));
		s.put(new Character(''),new Character('u'));
		s.put(new Character(''),new Character('u'));
		s.put(new Character(''),new Character('u'));
		s.put(new Character(''),new String("AE"));
		s.put(new Character(''),new String("Sh"));
		s.put(new Character(''),new String("Zh"));
		s.put(new Character(''),new String("ss"));
		s.put(new Character(''),new String("ae"));
		s.put(new Character(''),new String("sh"));
		s.put(new Character(''),new String("zh"));
		return s;
	}

	/**
	 * Transforms a dictionary written as text file to XML file.
	 * The output file differs for prefix, suffix and stem.
	 * For prefix and suffix, the output file is a list of &lt;entry&gt; tags.
	 * For stems, it is hierarchy of &lt;root&gt; tags, each containing  &lt;lemma&gt; tags.
	 *  &lt;root&gt; tags contain  &lt;entry&gt; tags.
	 * @param name the name of the dictionary. It is used to write the ouput file
	 * @param is InputStream for the text file.
	 * @throws RuntimeException when an IO error occurs reading from InputStream
	 * @see #transformDictionary2(String, InputStream)
	 */
	//TODO write comments to XML file
	private void transformDictionary(String name, InputStream is) throws RuntimeException{ //TODO : should be static
		Document xmlDoc = null;
		Element xmlRoot = null;
		Element rootElement = null;
		Element lemmaElement = null;
		Element deElement = null;
		try {
			// Create an XML Document
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbFactory.newDocumentBuilder();
			xmlDoc = docBuilder.newDocument();
			
			//xmlRoot = xmlDoc.createElement(name);
			xmlRoot = xmlDoc.createElement("dictionary");
			
			//HashSet lemmas = new HashSet();
			int forms = 0;
			String lemmaID = "";
			System.out.print("Transforming dictionary : " + name + " ");
			
			LineNumberReader IN = new LineNumberReader(new InputStreamReader(is,"ISO8859_1"));
			String line = null;
			while ((line = IN.readLine()) != null) {
				if ((IN.getLineNumber() % 1000) == 1) System.out.print(".");
				// new lemma
				if (line.startsWith(";; ")) {
					//append previous lemma
					if (lemmaElement != null)
						rootElement.appendChild(lemmaElement);
					//remove the beginning ;;
					lemmaID = line.substring(3);
					//arabize lemmaId
					lemmaID = AraMorph.arabizeWord(lemmaID);
					//write to XML
					lemmaElement = xmlDoc.createElement(LEMMA);
					lemmaElement.setAttribute(ID,lemmaID);
				}
				// root
				else if (line.startsWith(";--- ")) {
					//append previous root
					if (rootElement != null) {
						xmlRoot.appendChild(rootElement);
						if (lemmaElement != null)
							rootElement.appendChild(lemmaElement);
						lemmaElement = null;
					}
					//remove the beginning ;---
					String root = line.substring(5).trim();
					//remove any string after the root "see also ... etc."
					root = root.split(" ",2)[0].trim();
					//translate to arabic
					root = AraMorph.arabizeWord(root);
					rootElement = xmlDoc.createElement(ROOT);
					rootElement.setAttribute(VALUE,root);
				}
				//comment
				else if (line.startsWith(";")) {
					String comment = line.substring(1).trim();
					Comment commentElement = xmlDoc.createComment(comment);
					xmlRoot.appendChild(commentElement);
				} else {
					String split[] = line.split("\t",-1); //-1 to avoid triming of trail values
					
					//a little error-checking won't hurt :
					if (split.length != 4) {
						throw new RuntimeException("Entry in " + name + " (line " + IN.getLineNumber() + ") doesn't have 4 fields (3 tabs)");
					}
					String entry = AraMorph.arabizeWord(split[0]); // get the entry for use as key
					String vocalization = AraMorph.arabizeWord(split[1]);
					String morphology = split[2];
					String glossPOS = split[3];
					
					String gloss;
					String POS;
					
					//Pattern p;
					Matcher m;
					
					// two ways to get the POS info:
					// (1) explicitly, by extracting it from the gloss field:
					//p = Pattern.compile(".*" + "<pos>(.+?)</pos>" + ".*");
					m = p.matcher(glossPOS);
					if (m.matches()) {
						POS = m.group(1); //extract POS from glossPOS
						gloss = glossPOS; //we clean up the gloss later (see below)
					}
					// (2) by deduction: use the morphology (and sometimes the voc and gloss) to deduce the appropriate POS
					else {
						// we need the gloss to guess proper names
						gloss = glossPOS; 
						// null prefix or suffix
						if (morphology.matches("^(Pref-0|Suff-0)$")) { 
							POS = "";
						}
						else if (morphology.matches("^F" + ".*")) {
							POS = vocalization + "/FUNC_WORD";
						}
						else if (morphology.matches("^IV" + ".*")) {
							POS = vocalization + "/VERB_IMPERFECT";
						}
						else if (morphology.matches("^PV" + ".*")) {
							POS = vocalization + "/VERB_PERFECT";
						}
						else if (morphology.matches("^CV" + ".*")) {
							POS = vocalization + "/VERB_IMPERATIVE";
						}						
						else if (morphology.matches("^N" + ".*")) {
							// educated guess (99% correct)
							if (gloss.matches("^[A-Z]" + ".*")) {
								POS = vocalization + "/NOUN_PROP";
							}
							// (was NOUN_ADJ: some of these are really ADJ's and need to be tagged manually)
							else if (vocalization.matches(".*" + "iy~$")) { 
								POS = vocalization + "/NOUN";
							}
							else 
								POS = vocalization + "/NOUN";
						}
						else {
							throw new RuntimeException("No POS can be deduced in " + name + " (line " + IN.getLineNumber() + ")");
						}
					}
					
					// clean up the gloss: remove POS info and extra space, and convert upper-ASCII  to lower (it doesn't convert well to UTF-8)
					gloss = gloss.replaceFirst("<pos>.+?</pos>","");
					gloss = gloss.trim();
					
					gloss = translate(gloss);

					// note that although we read 4 fields from the dict we now save 5 fields in the hash table
					// because the info in last field, glossPOS, was split into two: gloss and POS
					DictionaryEntry de = new DictionaryEntry(entry, lemmaID, vocalization, morphology, gloss, POS);

					//add dictionary entry to xml file
					deElement = xmlDoc.createElement(ENTRY);
					//add unvocalized
					Element unvocalized = xmlDoc.createElement(UNVOCALIZED);
					unvocalized.appendChild(xmlDoc.createTextNode(de.getEntry()));
					deElement.appendChild(unvocalized);
					//add vocalized
					Element vocalized = xmlDoc.createElement(VOCALIZED);
					vocalized.appendChild(xmlDoc.createTextNode(de.getVocalization()));
					deElement.appendChild(vocalized);
					//add morphological category
					Element morphologicalCategory = xmlDoc.createElement(MORPHOLOGICAL_CATEGORY);
					morphologicalCategory.appendChild(xmlDoc.createTextNode(de.getMorphology()));
					deElement.appendChild(morphologicalCategory);
					//add glosses
					Element glosses = xmlDoc.createElement(GLOSSES);
					String[] glo = de.getGlosses();
					for (int i = 0; i < glo.length;i++) {
						Element glos = xmlDoc.createElement(GLOSS);
						glos.appendChild(xmlDoc.createTextNode(glo[i]));
						glosses.appendChild(glos);
					}
					deElement.appendChild(glosses);
					//add grammatical categories
					Element grammaticalCategories = xmlDoc.createElement(GRAMMATICAL_CATEGORIES);
					String[] pos = de.getPOS();
					for (int i = 0; i < pos.length;i++) {
						Element grammaticalCategory = xmlDoc.createElement(GRAMMATICAL_CATEGORY);
						String[] parts = pos[i].split("/");
						parts[0] = AraMorph.arabizeWord(parts[0]);
						String abc = parts[0];
						if (parts.length > 1)
							abc += "/"+parts[1];
						grammaticalCategory.appendChild(xmlDoc.createTextNode(abc));
						grammaticalCategories.appendChild(grammaticalCategory);
					}
					deElement.appendChild(grammaticalCategories);
					//append dictionary entry to xml file
					if (lemmaElement != null)
						lemmaElement.appendChild(deElement);
					else if (rootElement != null) {
						System.err.println("Found an entry without lemma in line: "+IN.getLineNumber());
						rootElement.appendChild(deElement);
					} else
						xmlRoot.appendChild(deElement);
					
					forms++;
				}
			}
			IN.close();
			if (rootElement != null) {
				xmlRoot.appendChild(rootElement);
				if (lemmaElement != null)
					rootElement.appendChild(lemmaElement);
				lemmaElement = null;
			}
			//write xml file
			xmlDoc.appendChild(xmlRoot);
			writeXML(xmlDoc,name+".xml");
			//write status message
			System.out.println();
			//if (!"".equals(lemmaID)) System.out.print(lemmas.size() + " lemmas and ");
			System.out.println(xmlRoot.getChildNodes().getLength() + " entries totalizing " + forms + " forms");
		}
		catch (IOException e) {
			throw new RuntimeException("Can not open : " + name);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Very similar to {@link #transformDictionary(String, InputStream)}, but writes Stems dictionary
	 * as a list of &lt;entry&gt; tags instead of a hierarchy structure.
	 * @param name the name of the dictionary. It is used to write the ouput file
	 * @param is InputStream for the text file.
	 * @throws RuntimeException when an IO error occurs reading from InputStream
	 * @see #transformDictionary(String, InputStream)
	 */
	private void transformDictionary2(String name, InputStream is) throws RuntimeException { //TODO : should be static
		Document xmlDoc = null;
		Element xmlRoot = null;
		Element deElement = null;
		
		try {
			// Create an XML Document
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbFactory.newDocumentBuilder();
			xmlDoc = docBuilder.newDocument();
			
			//xmlRoot = xmlDoc.createElement(name);
			xmlRoot = xmlDoc.createElement("dictionary");
			
			//HashSet lemmas = new HashSet();
			int forms = 0;
			String lemmaID = "";
			String root = "";
			System.out.print("Transforming dictionary : " + name + " ");
			
			LineNumberReader IN = new LineNumberReader(new InputStreamReader(is,"ISO8859_1"));
			String line = null;
			while ((line = IN.readLine()) != null) {
				if ((IN.getLineNumber() % 1000) == 1) System.out.print(".");
				// new lemma
				if (line.startsWith(";; ")) {
					//remove the beginning ;;
					lemmaID = line.substring(3);
					//arabize lemmaId
					lemmaID = AraMorph.arabizeWord(lemmaID);
				}
				// root
				else if (line.startsWith(";--- ")) {
					//remove the beginning ;---
					root = line.substring(5).trim();
					//remove any string after the root "see also ... etc."
					root = root.split(" ",2)[0].trim();
					//translate to arabic
					root = AraMorph.arabizeWord(root);
					//reset lemma
					lemmaID = "";
				}
				//comment
				else if (line.startsWith(";")) {
					String comment = line.substring(1).trim();
					Comment commentElement = xmlDoc.createComment(comment);
					xmlRoot.appendChild(commentElement);
				} else {
					String split[] = line.split("\t",-1); //-1 to avoid triming of trail values
					
					//a little error-checking won't hurt :
					if (split.length != 4) {
						throw new RuntimeException("Entry in " + name + " (line " + IN.getLineNumber() + ") doesn't have 4 fields (3 tabs)");
					}
					String entry = AraMorph.arabizeWord(split[0]); // get the entry for use as key
					String vocalization = AraMorph.arabizeWord(split[1]);
					String morphology = split[2];
					String glossPOS = split[3];
					
					String gloss;
					String POS;
					
					//Pattern p;
					Matcher m;
					
					// two ways to get the POS info:
					// (1) explicitly, by extracting it from the gloss field:
					//p = Pattern.compile(".*" + "<pos>(.+?)</pos>" + ".*");
					m = p.matcher(glossPOS);
					if (m.matches()) {
						POS = m.group(1); //extract POS from glossPOS
						gloss = glossPOS; //we clean up the gloss later (see below)
					}
					// (2) by deduction: use the morphology (and sometimes the voc and gloss) to deduce the appropriate POS
					else {
						// we need the gloss to guess proper names
						gloss = glossPOS; 
						// null prefix or suffix
						if (morphology.matches("^(Pref-0|Suff-0)$")) { 
							POS = "";
						}
						else if (morphology.matches("^F" + ".*")) {
							POS = vocalization + "/FUNC_WORD";
						}
						else if (morphology.matches("^IV" + ".*")) {
							POS = vocalization + "/VERB_IMPERFECT";
						}
						else if (morphology.matches("^PV" + ".*")) {
							POS = vocalization + "/VERB_PERFECT";
						}
						else if (morphology.matches("^CV" + ".*")) {
							POS = vocalization + "/VERB_IMPERATIVE";
						}						
						else if (morphology.matches("^N" + ".*")) {
							// educated guess (99% correct)
							if (gloss.matches("^[A-Z]" + ".*")) {
								POS = vocalization + "/NOUN_PROP";
							}
							// (was NOUN_ADJ: some of these are really ADJ's and need to be tagged manually)
							else if (vocalization.matches(".*" + "iy~$")) { 
								POS = vocalization + "/NOUN";
							}
							else 
								POS = vocalization + "/NOUN";
						}
						else {
							throw new RuntimeException("No POS can be deduced in " + name + " (line " + IN.getLineNumber() + ")");
						}
					}
					
					// clean up the gloss: remove POS info and extra space, and convert upper-ASCII  to lower (it doesn't convert well to UTF-8)
					gloss = gloss.replaceFirst("<pos>.+?</pos>","");
					gloss = gloss.trim();
					
					gloss = translate(gloss);
					
					// note that although we read 4 fields from the dict we now save 5 fields in the hash table
					// because the info in last field, glossPOS, was split into two: gloss and POS
					DictionaryEntry de = new DictionaryEntry(entry, lemmaID, vocalization, morphology, gloss, POS);
					
					//add dictionary entry to xml file
					deElement = xmlDoc.createElement(ENTRY);
					//add unvocalized
					Element unvocalized = xmlDoc.createElement(UNVOCALIZED);
					unvocalized.appendChild(xmlDoc.createTextNode(de.getEntry()));
					deElement.appendChild(unvocalized);
					//add vocalized
					Element vocalized = xmlDoc.createElement(VOCALIZED);
					vocalized.appendChild(xmlDoc.createTextNode(de.getVocalization()));
					deElement.appendChild(vocalized);
					//add morphological category
					Element morphologicalCategory = xmlDoc.createElement(MORPHOLOGICAL_CATEGORY);
					morphologicalCategory.appendChild(xmlDoc.createTextNode(de.getMorphology()));
					deElement.appendChild(morphologicalCategory);
					//add glosses
					Element glosses = xmlDoc.createElement(GLOSSES);
					String[] glo = de.getGlosses();
					for (int i = 0; i < glo.length;i++) {
						Element glos = xmlDoc.createElement(GLOSS);
						glos.appendChild(xmlDoc.createTextNode(glo[i]));
						glosses.appendChild(glos);
					}
					deElement.appendChild(glosses);
					//add grammatical categories
					Element grammaticalCategories = xmlDoc.createElement(GRAMMATICAL_CATEGORIES);
					String[] pos = de.getPOS();
					for (int i = 0; i < pos.length;i++) {
						Element grammaticalCategory = xmlDoc.createElement(GRAMMATICAL_CATEGORY);
						String[] parts = pos[i].split("/");
						parts[0] = AraMorph.arabizeWord(parts[0]);
						String abc = parts[0];
						if (parts.length > 1)
							abc += "/"+parts[1];
						grammaticalCategory.appendChild(xmlDoc.createTextNode(abc));
						grammaticalCategories.appendChild(grammaticalCategory);
					}
					deElement.appendChild(grammaticalCategories);
					//add root
					if (!root.equals("")) {
						Element rootElement = xmlDoc.createElement(ROOT);
						rootElement.appendChild(xmlDoc.createTextNode(root));
						deElement.appendChild(rootElement);
					}
					//add lemma
					if (!lemmaID.equals("")) {
						Element lemmaElement = xmlDoc.createElement(LEMMA);
						lemmaElement.appendChild(xmlDoc.createTextNode(lemmaID));
						deElement.appendChild(lemmaElement);
					} else if (!root.equals(""))
						System.err.println("Found an entry without lemma at line: "+IN.getLineNumber());

					//append dictionary entry to xml file
					xmlRoot.appendChild(deElement);
					
					forms++;
				}
			}
			IN.close();
			//write xml file
			xmlDoc.appendChild(xmlRoot);
			writeXML(xmlDoc,name+".xml");
			//write status message
			System.out.println();
			//if (!"".equals(lemmaID)) System.out.print(lemmas.size() + " lemmas and ");
			System.out.println(xmlRoot.getChildNodes().getLength() + " entries totalizing " + forms + " forms");
		}
		catch (IOException e) {
			throw new RuntimeException("Can not open : " + name);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}
	
	
	public void writeXML(Document xmldoc, String outputFilename) {
		try {
			StreamResult out = new StreamResult(outputFilename);
			DOMSource domSource = new DOMSource(xmldoc);
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer transformer = tf.newTransformer();
			// transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,
			// "yes");
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
			transformer.setOutputProperty(OutputKeys.ENCODING, XML_ENCODING);
			// we want to pretty format the XML output
			// note : this is broken in jdk1.5 beta!
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			//		
			transformer.transform(domSource, out);
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This function translates the given gloss according
	 * to the translate map. The characters of the gloss
	 * will be traversed one by one and replacted according
	 * to the translation map.
	 * @param gloss the gloss to be translated.
	 */
	private String translate(String gloss) {
		char[] translatedGloss = new char[gloss.length() * 2];
		int i=0;
		char[] arGloss = gloss.toCharArray();
		
		for (int x = 0;x < arGloss.length;x++) {
			Object result = translationMap.get(new Character(arGloss[x]));
			if (result == null)
				translatedGloss[i++] = arGloss[x];
			else if (result instanceof Character) {
				Character temp1 = (Character)result;
				translatedGloss[i++] = temp1.charValue();
			} else /*instance of String*/{
				String temp1 = (String)result;
				char temp2[] = temp1.toCharArray();
				for (int j = 0; j < temp2.length;j++)
					translatedGloss[i++]=temp2[j];
			}
		}
		return new String(translatedGloss,0,i);
	}

	/**
	 * This function transforms a compatability table from text format to XML file.
	 * @param firstPart name of the first part of each entry. Either "prefix", "suffix" or "stem"
	 * @param secondPart name of the first part of each entry. Either "prefix", "suffix" or "stem"
	 * @param name name of the compatability table. This name is used as the output file name.
	 * @param is InputStream for the text file to be converted.
	 * @throws RuntimeException if any IO Exception happened during reading out of the InputStream
	 */
	private void transformCompatibilityTable(String firstPart,String secondPart,String name, InputStream is) throws RuntimeException {
		System.out.print("Loading compatibility table : " + name + " ");
		Document xmlDoc = null;
		Element xmlRoot = null;
		try {
			// Create an XML Document
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbFactory.newDocumentBuilder();
			xmlDoc = docBuilder.newDocument();
			
			//xmlRoot = xmlDoc.createElement(name);
			xmlRoot = xmlDoc.createElement("compatability-table");
			
			LineNumberReader IN = new LineNumberReader(new InputStreamReader(is,"ISO8859_1"));
			String line = null;
			while ((line = IN.readLine()) != null) {
				if ((IN.getLineNumber() % 1000) == 1) System.out.print(".");
				if (!line.startsWith(";")) { //Ignore comments
					line = line.trim();
					line = line.replaceAll("\\s+", " ");
					String[] parts = line.split(" ");
					Element entry = xmlDoc.createElement("entry");
					Element first = xmlDoc.createElement(firstPart);
					first.appendChild(xmlDoc.createTextNode(parts[0]));
					Element second = xmlDoc.createElement(secondPart);
					second.appendChild(xmlDoc.createTextNode(parts[1]));
					entry.appendChild(first);
					entry.appendChild(second);
					xmlRoot.appendChild(entry);
					//set.add(line);
				} else {
					String comment = line.substring(1).trim();
					Comment commentElement = xmlDoc.createComment(comment);
					xmlRoot.appendChild(commentElement);
				}
			}
			IN.close();
			System.out.println();
			//System.out.println(set.size() + " entries");
			xmlDoc.appendChild(xmlRoot);
			writeXML(xmlDoc,name+".xml");
		}
		catch (IOException e) {
			throw new RuntimeException("Can not open : " + name);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
